/*
 * Copyright 2006 Integsystem Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package jp.integ.struts.lessconfig.config.rule.impl;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.factory.SingletonS2ContainerFactory;
import org.seasar.framework.convention.NamingConvention;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.lessconfig.config.rule.CommonNamingRule;

/**
 * パッケージルートの処理ルールクラスです。
 * 
 * @author Jun Futagawa (Integsystem Corporation)
 * @version $Date: 2006-09-20 01:02:28 +0900 $
 */
public class QualifiedRule {
	/** 大文字をパスに変換するためのパターンを表します。 */
	private static final Pattern upperNameToUriPattern = Pattern
			.compile("[0-9a-z_-]+|[A-Z][0-9a-z]+");

	/**
	 * Seasar2コンテナを取得します。
	 * 
	 * @return Seasar2コンテナ
	 */
	public static S2Container getContainer() {
		return SingletonS2ContainerFactory.getContainer();
	}

	/**
	 * NamingConventionを取得します。
	 * 
	 * @return NamingConvention
	 */
	public static NamingConvention getNamingConvention() {
		return (NamingConvention)getContainer().getComponent(
				NamingConvention.class);
	}

	/**
	 * ルートパッケージ名を取得します。
	 * 
	 * @return ルートパッケージ名
	 */
	public static String[] getRootPackageNames() {
		NamingConvention convention = getNamingConvention();
		String[] rootPackageNames = convention.getRootPackageNames().clone();
		int size = rootPackageNames.length;
		for (int i = 0; i < size; i++) {
			rootPackageNames[i] += "."
					+ convention.getSubApplicationRootPackageName();
		}
		return rootPackageNames;
	}

	/**
	 * 指定されたルートパッケージ名からアクションパスを取得します。
	 * 
	 * @param packageRule パッケージルール
	 * @param className アクションクラス名
	 * @param splitUpperName 大文字をパスに変換するかどうか
	 * @return アクションパス
	 */
	public static String getActionPath(String[] rootPackageNames,
			String className, boolean splitUpperName) {
		String actionPath = getActionPath(rootPackageNames, className);
		if (splitUpperName)
			actionPath = QualifiedRule.splitUpperName(actionPath);
		return actionPath;
	}

	/**
	 * 指定されたルートパッケージ名からアクションパスを取得します。
	 * 
	 * @param packageRule パッケージルール
	 * @param className アクションクラス名
	 * @return アクションパス
	 */
	public static String getActionPath(String[] rootPackageNames,
			String className) {
		String result = "";
		if (rootPackageNames != null && rootPackageNames.length > 0) {
			int length = rootPackageNames.length;
			for (int i = 0; i < length; i++) {
				String tmp = getActionPathBySingleRule(rootPackageNames[i],
						className);
				if (result.equals("") || tmp.length() < result.length()) {
					result = tmp;
				}
			}
		} else {
			result = getActionPathBySingleRule(null, className);
		}
		return result;
	}

	/**
	 * 指定されたルートパッケージ名からアクションパスを取得します。
	 * 
	 * @param rootPackageName
	 * @param className
	 * @return アクションパス
	 */
	public static String getActionPathBySingleRule(String rootPackageName,
			String className) {
		if (StringUtil.isEmpty(className))
			return null;
		if (!StringUtil.isEmpty(rootPackageName)) {
			className = className.replaceFirst(rootPackageName + ".", "");
		}
		className = "." + CommonNamingRule.decapitalizeName(className);
		className = getLastDecapitalizeName(className);
		className = className.replaceAll("\\.{2}?", "\\.").replaceAll("\\.",
				"/");
		return className.replaceFirst("Action$", "");
	}

	/**
	 * 大文字をパスに変換します。
	 * 
	 * @param uri パス表現
	 * @return 大文字をパスに変換したパス
	 */
	public static String splitUpperName(String uri) {
		if (StringUtil.isEmpty(uri))
			return null;
		int lastIndex = uri.length();
		Matcher matcher = upperNameToUriPattern.matcher(uri);
		StringBuffer buffer = new StringBuffer();
		while (matcher.find()) {
			String group = matcher.group().toLowerCase();
			if (matcher.end() == lastIndex) {
				buffer.append(group);
			} else {
				buffer.append(group).append("/");
			}
		}
		return "/" + buffer.toString();
	}

	/**
	 * 指定された名前の最後の . の次の1文字を小文字にします。
	 * 
	 * @param name
	 * @return
	 */
	private static String getLastDecapitalizeName(String name) {
		int index = name.lastIndexOf(".");
		if (index > 0 && name.length() > index) {
			index += 1;
			char chars[] = name.toCharArray();
			chars[index] = Character.toLowerCase(chars[index]);
			return new String(chars);
		}
		return name;
	}
}
