/*
 * Copyright 2006 Integsystem Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package jp.integ.struts.lessconfig.config.rule.impl;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import javax.servlet.ServletContext;
import org.apache.struts.action.ActionForward;
import org.apache.struts.config.ActionConfig;
import org.apache.struts.config.ForwardConfig;
import org.apache.struts.config.ModuleConfig;
import org.seasar.framework.log.Logger;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.Constants;
import org.seasar.struts.lessconfig.config.AutoStrutsConfigRule;
import org.seasar.struts.lessconfig.config.StrutsActionForwardConfig;
import org.seasar.struts.lessconfig.config.rule.impl.ZeroConfigActionRuleImpl;

/**
 * パッケージルート名ごとに自動アクション定義を設定するクラスです。
 * 
 * @author Jun Futagawa (Integsystem Corporation)
 * @version $Date: 2006-09-20 01:20:15 +0900 $
 */
public class QualifiedZeroConfigActionRuleImpl extends ZeroConfigActionRuleImpl {
	/** ロガーを表します。 */
	private static final Logger logger = Logger
			.getLogger(ZeroConfigActionRuleImpl.class);
	/** コンフィグルールを表します。。 */
	private AutoStrutsConfigRule configRule;
	/** クラス名の大文字をパスに変換するかどうかを表します。 */
	private boolean splitUpperName = true;
	/** ルートパッケージ名を表します。 */
	protected String[] rootPackageNames;

	/**
	 * 大文字をパスに変換するかどうかを判定します。
	 * 
	 * @return 大文字をパスに変換するかどうか
	 */
	public boolean isSplitUpperName() {
		return splitUpperName;
	}

	/**
	 * 大文字をパスに変換するかどうかを設定します。
	 * 
	 * @param splitUpperName 大文字をパスに変換するかどうか
	 */
	public void setSplitUpperName(boolean splitUpperName) {
		this.splitUpperName = splitUpperName;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getPath(Class actionClass, ModuleConfig config) {
		if (rootPackageNames == null) {
			rootPackageNames = QualifiedRule.getRootPackageNames();
		}
		String actionPath = QualifiedRule.getActionPath(rootPackageNames,
				actionClass.getName(), isSplitUpperName());
		return actionPath;
	}

	/**
	 * {@inheritDoc}
	 */
	public void addForwardConfig(Class actionClass, ActionConfig actionConfig,
			ServletContext servletContext) {
		ForwardConfig forwardConfig = actionConfig
				.findForwardConfig(Constants.SUCCESS);
		if (forwardConfig != null) {
			return;
		}
		String path = null;
		String[] viewExtension = this.configRule.getViewExtension();
		for (int i = 0; i < viewExtension.length; i++) {
			String file = getPath(actionClass, null) + "." + viewExtension[i];
			if (actionClass.getPackage() == null) {
				return;
			}
			String packageDir = "/"
					+ actionClass.getPackage().getName().replace('.', '/');
			path = getExistFilePath(this.configRule.getDocRoot(), packageDir,
					file, actionConfig, servletContext);
			String docRoot = this.configRule.getDocRoot();
			if (path == null) {
				path = getExistResourcePath(docRoot, packageDir, file,
						actionConfig, servletContext);
			}
			if (path != null) {
				addForwardConfig(path, actionConfig);
				return;
			}
		}
		String message = "View file was not found. "
				+ getPath(actionClass, null) + "."
				+ viewExtension[this.configRule.getViewExtension().length - 1];
		logger.info(message);
	}

	/**
	 * 指定されたパスと定義を持つフォワード定義を設定します。
	 * 
	 * @param path
	 * @param actionConfig
	 */
	private void addForwardConfig(String path, ActionConfig actionConfig) {
		ForwardConfig forwardConfig = new ActionForward();
		forwardConfig.setName(Constants.SUCCESS);
		forwardConfig.setPath(path);
		forwardConfig.setRedirect(StrutsActionForwardConfig.DEFAULT_REDIRECT);
		actionConfig.addForwardConfig(forwardConfig);
	}

	/**
	 * 存在するファイルパスを取得します。
	 * 
	 * @param docRoot
	 * @param packageDir
	 * @param file
	 * @param actionConfig
	 * @param servletContext
	 * @return
	 */
	private String getExistFilePath(String docRoot, String packageDir,
			String file, ActionConfig actionConfig,
			ServletContext servletContext) {
		String path = docRoot + packageDir + file;
		if (new File(servletContext.getRealPath(path)).exists()) {
			return path;
		} else if (StringUtil.isEmpty(packageDir)) {
			if (isLastExtension(file)
					&& actionConfig.findForwardConfigs().length == 0) {
				String message = "View file was not found."
						+ new File(path).getAbsolutePath();
				logger.info(message);
				// throw new IllegalStateException("View file was not found." +
				// new File(path).getAbsolutePath());
			}
			return null;
		} else {
			int index = packageDir.indexOf("/", 1);
			packageDir = packageDir.substring(index + 1);
			if (index < 0) {
				packageDir = "";
			} else {
				packageDir = "/" + packageDir;
			}
			return getExistFilePath(docRoot, packageDir, file, actionConfig,
					servletContext);
		}
	}

	/**
	 * 拡張子が有効なものか判断します。
	 * 
	 * @param file
	 * @return
	 */
	private boolean isLastExtension(String file) {
		return file.endsWith(this.configRule.getViewExtension()[this.configRule
				.getViewExtension().length - 1]);
	}

	/**
	 * {@inheritDoc}
	 */
	public void setAutoStrutsConfigPattern(AutoStrutsConfigRule configRule) {
		super.setAutoStrutsConfigPattern(configRule);
		this.configRule = configRule;
	}

	/**
	 * リソースが存在していれば、そのリソースを取得します。
	 * 
	 * @param docRoot
	 * @param packageDir
	 * @param file
	 * @param actionConfig
	 * @param servletContext
	 * @return
	 */
	private String getExistResourcePath(String docRoot, String packageDir,
			String file, ActionConfig actionConfig,
			ServletContext servletContext) {
		String path = docRoot + packageDir + file;
		URL resourceUrl;
		try {
			resourceUrl = servletContext.getResource(path);
		} catch (MalformedURLException e) {
			return null;
		}
		if (resourceUrl != null) {
			return path;
		}
		if (StringUtil.isEmpty(packageDir)) {
			return null;
		} else {
			int index = packageDir.indexOf("/", 1);
			packageDir = packageDir.substring(index + 1);
			if (index < 0) {
				packageDir = "";
			} else {
				packageDir = "/" + packageDir;
			}
			return getExistResourcePath(docRoot, packageDir, file,
					actionConfig, servletContext);
		}
	}
}
